"""
@file Reines_Interface.py
@brief Le programme résoud un échiquier avec une interface disponible.
@details Le programme applique l'algorithme de résolution BFS avec 3 types de grilles : 6x6 et 8x8 et 12x12 en plus d'afficher un rendu graphique.

@author TORRES Lucas
@version 0.4
@date 2024-03-20
"""
# Importation des modules nécessaires
import tkinter as tk
from tkinter import ttk, messagebox, Scrollbar
import time
from queue import Queue

# Vérifie si le placement d'une nouvelle reine 
def est_valide(placement, nouvelle_colonne):
    nouvelle_ligne = len(placement)  # La nouvelle ligne à tester
    for ligne in range(nouvelle_ligne):
        # Vérifie les colonnes et les diagonales 
        if placement[ligne] == nouvelle_colonne or \
           placement[ligne] - ligne == nouvelle_colonne - nouvelle_ligne or \
           placement[ligne] + ligne == nouvelle_colonne + nouvelle_ligne:
            return False
    return True

# Implémentation BFS
def bfs_n_reines(n):
    solutions = []  # Liste pour stocker les solutions
    file = Queue()  # File pour l'algorithme BFS
    file.put([])  # Initialisation avec un échiquier vide
    
    # Boucle tant que la file n'est pas vide
    while not file.empty():
        placement = file.get()
        
        # Si une solution complète est trouvée, l'ajoute à la liste des solutions
        if len(placement) == n:
            solutions.append(placement)
            continue
        
        # Teste chaque colonne 
        for colonne in range(n):
            if est_valide(placement, colonne):
                file.put(placement + [colonne])
                
    return solutions

# Affiche 
def afficher_solutions_gui(solutions, taille):
    window = tk.Toplevel()  # Crée une nouvelle fenêtre
    window.title(f"Solutions pour l'échiquier {taille}x{taille}")
    
    # Dimensions
    canvas_width = 400
    cell_size = canvas_width // taille
    canvas_height = canvas_width * len(solutions)
    
    # Configuration de la barre de défilement
    canvas = tk.Canvas(window, width=canvas_width + 20, height=400)
    canvas.pack(side=tk.LEFT, fill=tk.BOTH, expand=True)
    
    scrollbar = Scrollbar(window, orient="vertical", command=canvas.yview)
    scrollbar.pack(side=tk.RIGHT, fill="y")
    
    canvas.configure(yscrollcommand=scrollbar.set)
    canvas.bind('<Configure>', lambda e: canvas.configure(scrollregion=canvas.bbox("all")))
    
    frame_solutions = tk.Frame(canvas)
    canvas.create_window((0,0), window=frame_solutions, anchor="nw")
    
    # Dessin des solutions sur le l'échiquier
    for index, solution in enumerate(solutions, start=1):
        solution_canvas = tk.Canvas(frame_solutions, width=canvas_width, height=canvas_width, bg="white")
        solution_canvas.pack(pady=10)

        # Dessine l'échiquier place les reines
        for i in range(taille):
            for j in range(taille):
                color = "white" if (i + j) % 2 == 0 else "gray"
                solution_canvas.create_rectangle(j*cell_size, i*cell_size, (j+1)*cell_size, (i+1)*cell_size, fill=color)
                
                if j == solution[i]:
                    solution_canvas.create_text((j*cell_size+cell_size/2, i*cell_size+cell_size/2), text='Q', font=("Arial", int(cell_size/2)), fill="black")

# Interface principale pour choisir la taille de l'échiquier et afficher les solutions
def main_gui():
    main_window = tk.Tk()
    main_window.title("Résolveur d'échiquier des Reines")

    tk.Label(main_window, text="Choisissez la taille de l'échiquier:").pack()
    
    combo_taille = ttk.Combobox(main_window, values=[6, 8, 12])
    combo_taille.pack(pady=5)
    
    
    def on_submit():
        taille = int(combo_taille.get())
        temps_debut = time.time()
        solutions = bfs_n_reines(taille)
        temps_fin = time.time()
    
        messagebox.showinfo("Résultat", f"Nombre de solutions trouvées : {len(solutions)}\nTemps de résolution : {temps_fin - temps_debut:.4f} secondes.")
        
        afficher = messagebox.askyesno("Afficher les solutions", f"Voulez-vous afficher les solutions pour l'échiquier {taille}x{taille} ?")
        if afficher:
            afficher_solutions_gui(solutions, taille)
    
    ttk.Button(main_window, text="Soumettre", command=on_submit).pack(pady=5)
    
    main_window.mainloop()

if __name__ == "__main__":
    main_gui()
